#include <stdlib.h>
#include "ab.h"

//inicializa a rvore, fazendo a raiz igual a NULL
void cria(Arvore *A) {
     *A=NULL;
}

//verifica se a rvore est vazia
int IsEmpty(Arvore A) {
    if (A == NULL)
       return 1;
    else return 0;
}

//busca x na rvore, devendo iniciar com p=raiz ou outro ponto da rvore
no* busca(Arvore p, elem x) {
    no *aux;
    
    if (p==NULL)
       return(NULL);
    else if (p->info==x)
         return(p);
    else {
         aux=busca(p->esq,x);
         if (aux==NULL)
            aux=busca(p->dir,x);
         return(aux);
    }    
}



//se possvel, insere x  esquerda de pai na rvore
//se pai=-1, insere na raiz
void inserir_esq(Arvore *A, elem x, elem pai, int *erro) {
     no *aux, *p;

     if (pai!=-1) {
        aux=busca(*A,pai);
        if ((aux!=NULL) && (aux->esq==NULL)) {
           p=(no*) malloc(sizeof(no));
           p->info=x;
           p->esq=NULL;
           p->dir=NULL;
           aux->esq=p;
           *erro=0;
        }
        else *erro=1;
     }
     else if (*A==NULL) {
          p=(no*) malloc(sizeof(no));
          p->info=x;
          p->esq=NULL;
          p->dir=NULL;
          *A=p;
          *erro=0;
     }
     else *erro=1;
}

//se possvel, insere x  direita de pai na rvore
//se pai=-1, insere na raiz
void inserir_dir(Arvore *A, elem x, elem pai, int *erro) {
     no *aux, *p;
          
     if (pai!=-1) {
        aux=busca(*A,pai);
        if ((aux!=NULL) && (aux->dir==NULL)) {
           p=(no*) malloc(sizeof(no));
           p->info=x;
           p->esq=NULL;
           p->dir=NULL;
           aux->dir=p;
           *erro=0;
        }
        else *erro=1;
     }
     else if (*A==NULL) {
          p=(no*) malloc(sizeof(no));
          p->info=x;
          p->esq=NULL;
          p->dir=NULL;
          *A=p;
          *erro=0;
     }
     else *erro=1;
}

//imprime os elementos da rvore
void imprimir(Arvore p) {
     if (p!=NULL) {
        printf("%d(",p->info);
        imprimir(p->esq);
        printf(",");
        imprimir(p->dir);
        printf(")");
     }
     else printf("null");
}

//funo para computar a altura de uma rvore binria; devo comear com p apontando para a raiz
int altura(Arvore p) {
    int alt_esq, alt_dir;

    if (p==NULL)
       return(0);
    else {
         alt_esq=1+altura(p->esq);
         alt_dir=1+altura(p->dir);
         if (alt_esq>alt_dir)
            return(alt_esq);
         else return(alt_dir);
    }
}

// libera toda a memria usada na rvore
void finaliza(Arvore *p) {
     if (*p!=NULL) {
        finaliza(&(*p)->esq);
        finaliza(&(*p)->dir);
        free(*p);
     }
}



//funo que remove um elemento da rvore, caso ele esteja nela
 void remover(Arvore *A, elem x, int *erro) {
     no *p, *pai;
     int eh_filho_esq;
     
     //localizando o n e o pai dele
     if ((*A!=NULL) && ((*A)->info==x)) {
        pai=NULL;
        p=*A;
     }
     else {
          pai=busca_pai(*A,x);
          if (pai!=NULL) {
             if ((pai->esq!=NULL) && (pai->esq->info==x)) {
                p=pai->esq;
                eh_filho_esq=1;
             }
             else {
                  p=pai->dir;
                  eh_filho_esq=0;
             }
          }
          else p=NULL;
     }
     
     if (p==NULL)
        *erro=1;
     else {
          //primeiro caso: o n no tem filhos
          if ((p->esq==NULL) && (p->dir==NULL)) {
             if (pai!=NULL) {
                if (eh_filho_esq)
                   pai->esq=NULL;
                else pai->dir=NULL;
             }
             else *A=NULL;
             free(p);
             *erro=0;
          }
          //segundo caso: o n tem um ou dois filhos
          else {
               if (p->esq!=NULL) {
                  p->info=p->esq->info;
                  p->esq->info=x;
               }
               else {
                    p->info=p->dir->info;
                    p->dir->info=x;
               }
               remover(A,x,erro);
          }
     }
}


//funo de percurso de pre-ordem na rvore = busca em profundidade
void percurso_pre_ordem(Arvore p) {
     if (p!=NULL) {
        printf("%d\n",p->info);
        percurso_pre_ordem(p->esq);
        percurso_pre_ordem(p->dir);
     }
}

//funo de percurso de em-ordem na rvore
void percurso_em_ordem(Arvore p) {
     if (p!=NULL) {
        percurso_em_ordem(p->esq);
        printf("%d\n",p->info);        
        percurso_em_ordem(p->dir);
     }
}

//funo de percurso de pos-ordem na rvore
void percurso_pos_ordem(Arvore p) {
     if (p!=NULL) {
        percurso_pos_ordem(p->esq);
        percurso_pos_ordem(p->dir);
        printf("%d\n",p->info);        
     }
}
